@@ -9,8 +9,9 @@ import (
 // ValidateHosts performs remote OS detection
 type ValidateHosts struct {
 	GenericPhase
-	hncount        map[string]int
-	machineidcount map[string]int
+	hncount          map[string]int
+	machineidcount   map[string]int
+	privateaddrcount map[string]int
 }
 
 // Title for the phase
@@ -22,12 +23,21 @@ func (p *ValidateHosts) Title() string {
 func (p *ValidateHosts) Run() error {
 	p.hncount = make(map[string]int, len(p.Config.Spec.Hosts))
 	p.machineidcount = make(map[string]int, len(p.Config.Spec.Hosts))
+	p.privateaddrcount = make(map[string]int, len(p.Config.Spec.Hosts))
+
 	for _, h := range p.Config.Spec.Hosts {
 		p.hncount[h.Metadata.Hostname]++
 		p.machineidcount[h.Metadata.MachineID]++
+		p.privateaddrcount[h.PrivateAddress]++
 	}
 
-	return p.parallelDo(p.Config.Spec.Hosts, p.validateUniqueHostname, p.validateUniqueMachineID, p.validateSudo)
+	return p.parallelDo(
+		p.Config.Spec.Hosts,
+		p.validateUniqueHostname,
+		p.validateUniqueMachineID,
+		p.validateUniquePrivateAddress,
+		p.validateSudo,
+	)
 }
 
 func (p *ValidateHosts) validateUniqueHostname(h *cluster.Host) error {
@@ -38,6 +48,14 @@ func (p *ValidateHosts) validateUniqueHostname(h *cluster.Host) error {
 	return nil
 }
 
+func (p *ValidateHosts) validateUniquePrivateAddress(h *cluster.Host) error {
+	if p.privateaddrcount[h.PrivateAddress] > 1 {
+		return fmt.Errorf("privateAddress %s is not unique: %s", h.PrivateAddress, h.Metadata.Hostname)
+	}
+
+	return nil
+}
+
 func (p *ValidateHosts) validateUniqueMachineID(h *cluster.Host) error {
 	if p.machineidcount[h.Metadata.MachineID] > 1 {
 		return fmt.Errorf("machine id %s is not unique: %s", h.Metadata.MachineID, h.Metadata.Hostname)
