@@ -4,11 +4,13 @@ import (
 	"encoding/json"
 	"fmt"
 	gos "os"
+	"path/filepath"
 	"regexp"
 	"strconv"
 	"strings"
 	"time"
 
+	"github.com/alessio/shellescape"
 	"github.com/avast/retry-go"
 	"github.com/creasty/defaults"
 	validation "github.com/go-ozzo/ozzo-validation/v4"
@@ -30,6 +32,7 @@ type Host struct {
 	Reset            bool              `yaml:"reset,omitempty"`
 	PrivateInterface string            `yaml:"privateInterface,omitempty"`
 	PrivateAddress   string            `yaml:"privateAddress,omitempty"`
+	DataDir          string            `yaml:"dataDir,omitempty"`
 	Environment      map[string]string `yaml:"environment,flow,omitempty"`
 	UploadBinary     bool              `yaml:"uploadBinary,omitempty"`
 	K0sBinaryPath    string            `yaml:"k0sBinaryPath,omitempty"`
@@ -64,6 +67,14 @@ func (h *Host) SetDefaults() {
 	if h.InstallFlags.Get("--no-taints") != "" && h.InstallFlags.GetValue("--no-taints") != "false" {
 		h.NoTaints = true
 	}
+
+	if dd := h.InstallFlags.Get("--data-dir"); dd != "" {
+		if h.DataDir != "" {
+			log.Debugf("%s: changed dataDir from '%s' to '%s' because of --data-dir installFlag", h, h.DataDir, dd)
+		}
+		h.InstallFlags.Delete("--data-dir")
+		h.DataDir = dd
+	}
 }
 
 func (h *Host) Validate() error {
@@ -93,6 +104,7 @@ type configurer interface {
 	K0sBinaryPath() string
 	K0sBinaryVersion(os.Host) (*version.Version, error)
 	K0sConfigPath() string
+	DataDirDefaultPath() string
 	K0sJoinTokenPath() string
 	WriteFile(os.Host, string, string, string) error
 	UpdateEnvironment(os.Host, map[string]string) error
@@ -102,7 +114,7 @@ type configurer interface {
 	ReadFile(os.Host, string) (string, error)
 	FileExist(os.Host, string) bool
 	Chmod(os.Host, string, string, ...exec.Option) error
-	DownloadK0s(os.Host, *version.Version, string) error
+	DownloadK0s(os.Host, string, *version.Version, string) error
 	DownloadURL(os.Host, string, string, ...exec.Option) error
 	InstallPackage(os.Host, ...string) error
 	FileContains(os.Host, string, string) bool
@@ -133,6 +145,7 @@ type configurer interface {
 // HostMetadata resolved metadata for host
 type HostMetadata struct {
 	K0sBinaryVersion  string
+	K0sBinaryTempFile string
 	K0sRunningVersion string
 	Arch              string
 	IsK0sLeader       bool
@@ -153,7 +166,7 @@ func (h *Host) UnmarshalYAML(unmarshal func(interface{}) error) error {
 		return err
 	}
 
-	if h.SSH.HostKey != "" {
+	if h.SSH != nil && h.SSH.HostKey != "" {
 		log.Warnf("%s: host.ssh.hostKey is deprecated, use a ssh known hosts file instead", h)
 	}
 
@@ -245,6 +258,8 @@ func (h *Host) K0sInstallCommand() (string, error) {
 	role := h.Role
 	flags := h.InstallFlags
 
+	flags.AddOrReplace(fmt.Sprintf("--data-dir=%s", h.DataDir))
+
 	switch role {
 	case "controller+worker":
 		role = "controller"
@@ -298,12 +313,12 @@ func (h *Host) K0sInstallCommand() (string, error) {
 
 // K0sBackupCommand returns a full command to be used as run k0s backup
 func (h *Host) K0sBackupCommand(targetDir string) string {
-	return h.Configurer.K0sCmdf("backup --save-path %s", targetDir)
+	return h.Configurer.K0sCmdf("backup --save-path %s --data-dir %s", shellescape.Quote(targetDir), h.DataDir)
 }
 
 // K0sRestoreCommand returns a full command to restore cluster state from a backup
 func (h *Host) K0sRestoreCommand(backupfile string) string {
-	return h.Configurer.K0sCmdf("restore %s", backupfile)
+	return h.Configurer.K0sCmdf("restore --data-dir=%s %s", h.DataDir, shellescape.Quote(backupfile))
 }
 
 // IsController returns true for controller and controller+worker roles
@@ -321,19 +336,28 @@ func (h *Host) K0sServiceName() string {
 	}
 }
 
-// UpdateK0sBinary updates the binary on the host either by downloading or uploading, based on the config
-func (h *Host) UpdateK0sBinary(version *version.Version) error {
-	if h.UploadBinaryPath != "" {
-		if err := h.Upload(h.UploadBinaryPath, h.Configurer.K0sBinaryPath(), exec.Sudo(h)); err != nil {
-			return err
-		}
-		if err := h.Configurer.Chmod(h, h.Configurer.K0sBinaryPath(), "0700", exec.Sudo(h)); err != nil {
-			return err
-		}
-	} else {
-		if err := h.Configurer.DownloadK0s(h, version, h.Metadata.Arch); err != nil {
-			return err
-		}
+// InstallK0sBinary installs the k0s binary from the provided file path to K0sBinaryPath
+func (h *Host) InstallK0sBinary(path string) error {
+	if !h.Configurer.FileExist(h, path) {
+		return fmt.Errorf("k0s binary tempfile not found")
+	}
+
+	dir := filepath.Dir(h.Configurer.K0sBinaryPath())
+	if err := h.Execf(`install -m 0755 -o root -g root -d "%s"`, dir, exec.Sudo(h)); err != nil {
+		return fmt.Errorf("create k0s binary dir: %w", err)
+	}
+
+	if err := h.Execf(`install -m 0750 -o root -g root "%s" "%s"`, path, h.Configurer.K0sBinaryPath(), exec.Sudo(h)); err != nil {
+		return fmt.Errorf("install k0s binary: %w", err)
+	}
+
+	return nil
+}
+
+// UpdateK0sBinary updates the binary on the host from the provided file path
+func (h *Host) UpdateK0sBinary(path string, version *version.Version) error {
+	if err := h.InstallK0sBinary(path); err != nil {
+		return fmt.Errorf("update k0s binary: %w", err)
 	}
 
 	updatedVersion, err := h.Configurer.K0sBinaryVersion(h)
@@ -362,7 +386,7 @@ type kubeNodeStatus struct {
 
 // KubeNodeReady runs kubectl on the host and returns true if the given node is marked as ready
 func (h *Host) KubeNodeReady() (bool, error) {
-	output, err := h.ExecOutput(h.Configurer.KubectlCmdf(h, "get node -l kubernetes.io/hostname=%s -o json", h.Metadata.Hostname), exec.HideOutput(), exec.Sudo(h))
+	output, err := h.ExecOutput(h.Configurer.KubectlCmdf(h, "get node --data-dir=%s -l kubernetes.io/hostname=%s -o json", h.DataDir, h.Metadata.Hostname), exec.HideOutput(), exec.Sudo(h))
 	if err != nil {
 		return false, err
 	}
@@ -407,17 +431,17 @@ func (h *Host) WaitKubeNodeReady() error {
 
 // DrainNode drains the given node
 func (h *Host) DrainNode(node *Host) error {
-	return h.Exec(h.Configurer.KubectlCmdf(h, "drain --grace-period=120 --force --timeout=5m --ignore-daemonsets --delete-local-data %s", node.Metadata.Hostname), exec.Sudo(h))
+	return h.Exec(h.Configurer.KubectlCmdf(h, "drain --data-dir=%s --grace-period=120 --force --timeout=5m --ignore-daemonsets --delete-emptydir-data %s", h.DataDir, node.Metadata.Hostname), exec.Sudo(h))
 }
 
 // UncordonNode marks the node schedulable again
 func (h *Host) UncordonNode(node *Host) error {
-	return h.Exec(h.Configurer.KubectlCmdf(h, "uncordon %s", node.Metadata.Hostname), exec.Sudo(h))
+	return h.Exec(h.Configurer.KubectlCmdf(h, "uncordon --data-dir=%s %s", h.DataDir, node.Metadata.Hostname), exec.Sudo(h))
 }
 
 // DeleteNode deletes the given node from kubernetes
 func (h *Host) DeleteNode(node *Host) error {
-	return h.Exec(h.Configurer.KubectlCmdf(h, "delete node %s", node.Metadata.Hostname), exec.Sudo(h))
+	return h.Exec(h.Configurer.KubectlCmdf(h, "delete node --data-dir=%s %s", h.DataDir, node.Metadata.Hostname), exec.Sudo(h))
 }
 
 func (h *Host) LeaveEtcd(node *Host) error {
@@ -425,7 +449,7 @@ func (h *Host) LeaveEtcd(node *Host) error {
 	if node.PrivateAddress != "" {
 		etcdAddress = node.PrivateAddress
 	}
-	return h.Exec(h.Configurer.K0sCmdf("etcd leave --peer-address %s", etcdAddress), exec.Sudo(h))
+	return h.Exec(h.Configurer.K0sCmdf("etcd leave --peer-address %s --datadir %s", etcdAddress, h.DataDir), exec.Sudo(h))
 }
 
 // CheckHTTPStatus will perform a web request to the url and return an error if the http status is not the expected
